<stylesheet
	xmlns="http://www.w3.org/1999/XSL/Transform"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:jbg="http://jbg.sf.net/jbg"
	version="2.0">
<import href="functions/JbgBusinessFunctions.xsl" />
<output method="text" />

<template match="//jbg:beans/jbg:bean">
	<variable name="bean" select="." />
	<variable name="bean-class" select="seaf:type-class(@class)" />
	<variable name="bean-dir" select="seaf:type-dir(@class)" />
	<variable name="all-properties" 
		select="jbg:all-bean-properties(.)[not(@status='supplementary')]" />
	<variable name="non-final-accessors" select="ancestor::jbg:beans/@final-accessors='no'" />
	<variable name="to-string-all" select="ancestor::jbg:beans/@to-string-properties='all'" />
	<variable name="import-simple-converters" 
		select="not(empty(.//jbg:property/jbg:simple-type[jbg:is-temporal(@type)]))" />
	<!-- Trace the names of files generated: 
	<message select="concat('Creating JavaBean ',$bean-dir,'/',$bean-class,'.java')" />
	 -->

<result-document href="{$bean-dir}/{$bean-class}.java">

	<value-of select="jbg:bean-class-start(@class,$to-string-all,
		$import-simple-converters,jbg:description,@assume-instance-of,(@final='yes'),
		@extends,@implements,jbg:annotation,@serializable-with-id)"/>

	<!-- Property declarations -->
	<for-each select="jbg:property">
		<variable name="name" select="@name" />
		<variable name="type" select="jbg:property-type(.)" />
		<variable name="initial-value" select="jbg:collection-initial-value(.)"/>

		<value-of select="jbg:attribute-declaration($name,$type,
			$initial-value,jbg:description,jbg:annotation)" />
	</for-each>

	<!-- Constructors -->
	<value-of select="jbg:constructor-declaration($bean-class,(),'Default')"/>
	<if test="$all-properties[jbg:bean-type[not(@treat-as-simple=true())] or jbg:sequence] and 
			jbg:simple-properties($all-properties)">
		<value-of select="jbg:constructor-declaration($bean-class,
			jbg:simple-properties($all-properties),'Simple properties')"/>
	</if>
	<if test="$all-properties">
		<value-of select="jbg:constructor-declaration($bean-class,$all-properties,'Full')"/>
	</if>

	<!-- Getters / setters -->
	<for-each select="jbg:property">
		<variable name="name" select="@name" />
		<variable name="type" select="jbg:property-type(.)" />
		<variable name="reverse-single-property" 
			select="jbg:bean-type/@reverse-property-name" />
		<variable name="separate-reverse-setter" 
			select="not(jbg:bean-type/@separate-reverse-setter='false')"/>
		<variable name="reverse-col-property"
			select="jbg:sequence[jbg:is-sequence-collection(.)]/jbg:bean-type/@reverse-property-name" />
		<variable name="reverse-col-single-type">
			<if test="jbg:sequence[jbg:is-sequence-collection(.)]">
				<value-of select="jbg:property-type(jbg:sequence)" />
			</if>
		</variable>
		<variable name="not-null-value" select="jbg:collection-initial-value(.)"/>
		<variable name="chained-as-type">
			<if test="ancestor::jbg:beans/@chainable-setters='yes'">
				<value-of select="$bean-class"/>
			</if>
		</variable>

		<value-of select="seaf:getter-setter-declaration(
			$name,$type,$reverse-single-property,$separate-reverse-setter,
			$reverse-col-property,$reverse-col-single-type,$not-null-value,
			$chained-as-type,$bean/@assume-instance-of,$non-final-accessors)" />
		<if test="jbg:sequence[jbg:is-sequence-collection(.)]">
			<value-of select="jbg:collection-setter-helpers(
				$name,$reverse-col-single-type,$reverse-col-property,
				$bean/@assume-instance-of,$non-final-accessors)" />
		</if>
	</for-each>

	// Java standard methods implementation<text />

	<variable name="business-key-properties" select="jbg:business-key-properties(.)" />
	<value-of select="jbg:get-business-key-values-method($business-key-properties)"/>
	<value-of select="jbg:hash-code-method($bean-class)"/>
	<value-of select="jbg:equals-method($bean-class)"/>

	<variable name="to-string-properties" select="jbg:to-string-properties(.)" />
	<value-of select="jbg:to-string-method($bean-class,$to-string-properties)"/>

	<if test="jbg:additional-code">
		<value-of select="jbg:additional-code"/>
	</if>

	<value-of select="jbg:serial-uid(@serializable-with-id)"/>

}
<text />

</result-document>
</template>

</stylesheet>
